% calculate charge and energy
function [charge, wtotal] = calcchar(Parm, Mesh, idVolt, Epsr, V)
	% mesh
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;
	%dxc = Mesh.dxc';  % row vector
	dxc = Mesh.dxc;
	dyc = Mesh.dyc;
	dzc = Mesh.dzc;
	rxp = Mesh.rxp;
	rxm = Mesh.rxm;
	ryp = Mesh.ryp;
	rym = Mesh.rym;
	rzp = Mesh.rzp;
	rzm = Mesh.rzm;

	% total energy [pW] (exclude boundary)
	wtotal = 0;
	for k = 2 : nz - 1
	for j = 2 : ny - 1
	for i = 2 : nx - 1
		ex = -(V(i + 1, j,     k    ) - V(i - 1, j,     k    )) / (2 * dxc(i));
		ey = -(V(i,     j + 1, k    ) - V(i,     j - 1, k    )) / (2 * dyc(j));
		ez = -(V(i,     j,     k + 1) - V(i,     j,     k - 1)) / (2 * dzc(k));
		dv = dxc(i) * dyc(j) * dzc(k);
		wtotal = wtotal + (Epsr(i, j, k) * (ex^2 + ey^2 + ez^2) * dv);
	end
	end
	end
	wtotal = 0.5 * Parm.eps0 * wtotal;

	% Q [C/m^3]
	charge = zeros(1, length(Parm.volt));
	%for k = 2 : nz - 1
	%for j = 2 : ny - 1
	%for i = 2 : nx - 1
	for k = 1 : nz
	for j = 1 : ny
	for i = 1 : nx
		if (idVolt(i, j, k) > 0)
			%{
			v = V(i, j, k);
			e = Epsr(i, j, k);
			q = (v - V(i + 1, j,     k    )) * (e + Epsr(i + 1, j,     k    )) / 2 * rxp(i) + ...
			    (v - V(i - 1, j,     k    )) * (e + Epsr(i - 1, j,     k    )) / 2 * rxm(i) + ...
			    (v - V(i,     j + 1, k    )) * (e + Epsr(i,     j + 1, k    )) / 2 * ryp(j) + ...
			    (v - V(i,     j - 1, k    )) * (e + Epsr(i,     j - 1, k    )) / 2 * rym(j) + ...
			    (v - V(i,     j,     k + 1)) * (e + Epsr(i,     j,     k + 1)) / 2 * rzp(k) + ...
			    (v - V(i,     j,     k - 1)) * (e + Epsr(i,     j,     k - 1)) / 2 * rzm(k);
			%}
			[xp, xm, yp, ym, zp, zm] = average(V, Epsr, i, j, k, nx, ny, nz);
			q = (xp * rxp(i)) + (xm * rxm(i)) + (yp * ryp(j)) + (ym * rym(j)) + (zp * rzp(k)) + (zm * rzm(k));
			dv = dxc(i) * dyc(j) * dzc(k);
			charge(idVolt(i, j, k)) = charge(idVolt(i, j, k)) + (q * dv);
		end
	end
	end
	end
	charge = Parm.eps0 * charge;
end

% charge factor
function [xp, xm, yp, ym, zp, zm] = average(v, e, i, j, k, nx, ny, nz)
	xp = 0; xm = 0; yp = 0; ym = 0; zp = 0; zm = 0;

	% X+
	if (1 <= i) && (i < nx)
		xp = (v(i,     j,     k    ) - v(i + 1, j,     k    )) ...
		   * (e(i,     j,     k    ) + e(i + 1, j,     k    )) / 2;
	end

	% X-
	if (1 < i) && (i <= nx)
		xm = (v(i,     j,     k    ) - v(i - 1, j,     k    )) ...
		   * (e(i,     j,     k    ) + e(i - 1, j,     k    )) / 2;
	end

	% Y+
	if (1 <= j) && (j < ny)
		yp = (v(i,     j,     k    ) - v(i,     j + 1, k    )) ...
		   * (e(i,     j,     k    ) + e(i,     j + 1, k    )) / 2;
	end

	% Y-
	if (1 < j) && (j <= ny)
		ym = (v(i,     j,     k    ) - v(i,     j - 1, k    )) ...
		   * (e(i,     j,     k    ) + e(i,     j - 1, k    )) / 2;
	end

	% Z+
	if (1 <= k) && (k < nz)
		zp = (v(i,     j,     k    ) - v(i,     j,     k + 1)) ...
		   * (e(i,     j,     k    ) + e(i,     j,     k + 1)) / 2;
	end

	% Z-
	if (1 < k) && (k <= nz)
		zm = (v(i,     j,     k    ) - v(i,     j,     k - 1)) ...
		   * (e(i,     j,     k    ) + e(i,     j,     k - 1)) / 2;
	end
end
