% make 3D lines from geometry data
function [gline, pline, nline] = geomlines(Geom)
	nc = 36;
	nline = alloc_lines(Geom, nc);
	[gline, pline, nline] = make_lines(Geom, nline, nc);
end

% make line data
function [g, p, n] = make_lines(Geom, nline, nc)
	% alloc
	g = zeros(nline, 2, 3);
	p = zeros(1, nline);

	n = 0;
	for ig = 1 : length(Geom)
		type  = Geom(ig).type;
		shape = Geom(ig).shape;
		pos   = Geom(ig).pos;

		x1 = pos(1);
		x2 = pos(2);
		y1 = pos(3);
		y2 = pos(4);
		z1 = pos(5);
		z2 = pos(6);

		x0 = (x1 + x2) / 2;
		y0 = (y1 + y2) / 2;
		z0 = (z1 + z2) / 2;

		if      (shape == 1)
			% rectangle
			dx = abs(x2 - x1);
			dy = abs(y2 - y1);
			dz = abs(z2 - z1);
			% wire
			if     (dy < eps) && (dz < eps)
				[g, p, n] = line_data(g, p, n, type, x1, y1, z1, x2, y1, z1);
			elseif (dz < eps) && (dx < eps)
				[g, p, n] = line_data(g, p, n, type, x1, y1, z1, x1, y2, z1);
			elseif (dx < eps) && (dy < eps)
				[g, p, n] = line_data(g, p, n, type, x1, y1, z1, x1, y1, z2);
			% plane
			elseif (dx < eps)
				[g, p, n] = rectangle_data(g, p, n, type, 'X', x1, y1, z1, y2, z2);
			elseif (dy < eps)
				[g, p, n] = rectangle_data(g, p, n, type, 'Y', y1, z1, x1, z2, x2);
			elseif (dz < eps)
				[g, p, n] = rectangle_data(g, p, n, type, 'Z', z1, x1, y1, x2, y2);
			%box
			else
				[g, p, n] = rectangle_data(g, p, n, type, 'Z', z1, x1, y1, x2, y2);
				[g, p, n] = rectangle_data(g, p, n, type, 'Z', z2, x1, y1, x2, y2);
				[g, p, n] = line_data(g, p, n, type, x1, y1, z1, x1, y1, z2);
				[g, p, n] = line_data(g, p, n, type, x2, y1, z1, x2, y1, z2);
				[g, p, n] = line_data(g, p, n, type, x2, y2, z1, x2, y2, z2);
				[g, p, n] = line_data(g, p, n, type, x1, y2, z1, x1, y2, z2);
			end
		elseif (shape == 2)
			% sphere
			[g, p, n] = ellipse_data(g, p, n, type, 'X', x0, y1, z1, y2, z2, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'Y', y0, z1, x1, z2, x2, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'Z', z0, x1, y1, x2, y2, nc);
		elseif (shape == 11)
			% X-cylinder
			[g, p, n] = ellipse_data(g, p, n, type, 'X', x1, y1, z1, y2, z2, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'X', x2, y1, z1, y2, z2, nc);
			[g, p, n] = line_data(g, p, n, type, x1, y1, z0, x2, y1, z0);
			[g, p, n] = line_data(g, p, n, type, x1, y2, z0, x2, y2, z0);
			[g, p, n] = line_data(g, p, n, type, x1, y0, z1, x2, y0, z1);
			[g, p, n] = line_data(g, p, n, type, x1, y0, z2, x2, y0, z2);
		elseif (shape == 12)
			% Y-cylinder
			[g, p, n] = ellipse_data(g, p, n, type, 'Y', y1, z1, x1, z2, x2, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'Y', y2, z1, x1, z2, x2, nc);
			[g, p, n] = line_data(g, p, n, type, x0, y1, z1, x0, y2, z1);
			[g, p, n] = line_data(g, p, n, type, x0, y1, z2, x0, y2, z2);
			[g, p, n] = line_data(g, p, n, type, x1, y1, z0, x1, y2, z0);
			[g, p, n] = line_data(g, p, n, type, x2, y1, z0, x2, y2, z0);
		elseif (shape == 13)
			% Z-cylinder
			[g, p, n] = ellipse_data(g, p, n, type, 'Z', z1, x1, y1, x2, y2, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'Z', z2, x1, y1, x2, y2, nc);
			[g, p, n] = line_data(g, p, n, type, x1, y0, z1, x1, y0, z2);
			[g, p, n] = line_data(g, p, n, type, x2, y0, z1, x2, y0, z2);
			[g, p, n] = line_data(g, p, n, type, x0, y1, z1, x0, y1, z2);
			[g, p, n] = line_data(g, p, n, type, x0, y2, z1, x0, y2, z2);
		elseif (shape == 31)
			% X-pillar
			px = pos(1:2);
			py = pos(3:5);
			pz = pos(6:8);
			for i = 1: 2
				[g, p, n] = line_data(g, p, n, type, px(i), py(1), pz(1), px(i), py(2), pz(2));
				[g, p, n] = line_data(g, p, n, type, px(i), py(2), pz(2), px(i), py(3), pz(3));
				[g, p, n] = line_data(g, p, n, type, px(i), py(3), pz(3), px(i), py(1), pz(1));
			end
			for i = 1 : 3
				[g, p, n] = line_data(g, p, n, type, px(1), py(i), pz(i), px(2), py(i), pz(i));
			end
		elseif (shape == 32)
			% Y-pillar
			py = pos(1:2);
			pz = pos(3:5);
			px = pos(6:8);
			for i = 1 : 2
				[g, p, n] = line_data(g, p, n, type, px(1), py(i), pz(1), px(2), py(i), pz(2));
				[g, p, n] = line_data(g, p, n, type, px(2), py(i), pz(2), px(3), py(i), pz(3));
				[g, p, n] = line_data(g, p, n, type, px(3), py(i), pz(3), px(1), py(i), pz(1));
			end
			for i = 1 : 3
				[g, p, n] = line_data(g, p, n, type, px(i), py(1), pz(i), px(i), py(2), pz(i));
			end
		elseif (shape == 33)
			% Z-pillar
			pz = pos(1:2);
			px = pos(3:5);
			py = pos(6:8);
			for i = 1 : 2
				[g, p, n] = line_data(g, p, n, type, px(1), py(1), pz(i), px(2), py(2), pz(i));
				[g, p, n] = line_data(g, p, n, type, px(2), py(2), pz(i), px(3), py(3), pz(i));
				[g, p, n] = line_data(g, p, n, type, px(3), py(3), pz(i), px(1), py(1), pz(i));
			end
			for i = 1 : 3
				[g, p, n] = line_data(g, p, n, type, px(i), py(i), pz(1), px(i), py(i), pz(2));
			end
		elseif (shape == 41)
			% X-pyramid
			x1 = pos(1);
			x2 = pos(2);
			y0 = pos(3);
			z0 = pos(4);
			h1y = pos(5) / 2;
			h1z = pos(6) / 2;
			h2y = pos(7) / 2;
			h2z = pos(8) / 2;
			[g, p, n] = rectangle_data(g, p, n, type, 'X', x1, y0 - h1y, z0 - h1z, y0 + h1y, z0 + h1z);
			[g, p, n] = rectangle_data(g, p, n, type, 'X', x2, y0 - h2y, z0 - h2z, y0 + h2y, z0 + h2z);
			[g, p, n] = line_data(g, p, n, type, x1, y0 - h1y, z0 - h1z, x2, y0 - h2y, z0 - h2z);
			[g, p, n] = line_data(g, p, n, type, x1, y0 - h1y, z0 + h1z, x2, y0 - h2y, z0 + h2z);
			[g, p, n] = line_data(g, p, n, type, x1, y0 + h1y, z0 - h1z, x2, y0 + h2y, z0 - h2z);
			[g, p, n] = line_data(g, p, n, type, x1, y0 + h1y, z0 + h1z, x2, y0 + h2y, z0 + h2z);
		elseif (shape == 42)
			% Y-pyramid
			y1 = pos(1);
			y2 = pos(2);
			z0 = pos(3);
			x0 = pos(4);
			h1z = pos(5) / 2;
			h1x = pos(6) / 2;
			h2z = pos(7) / 2;
			h2x = pos(8) / 2;
			[g, p, n] = rectangle_data(g, p, n, type, 'Y', y1, z0 - h1z, x0 - h1x, z0 + h1z, x0 + h1x);
			[g, p, n] = rectangle_data(g, p, n, type, 'Y', y2, z0 - h2z, x0 - h2x, z0 + h2z, x0 + h2x);
			[g, p, n] = line_data(g, p, n, type, x0 - h1x, y1, z0 - h1z, x0 - h2x, y2, z0 - h2z);
			[g, p, n] = line_data(g, p, n, type, x0 - h1x, y1, z0 + h1z, x0 - h2x, y2, z0 + h2z);
			[g, p, n] = line_data(g, p, n, type, x0 + h1x, y1, z0 - h1z, x0 + h2x, y2, z0 - h2z);
			[g, p, n] = line_data(g, p, n, type, x0 + h1x, y1, z0 + h1z, x0 + h2x, y2, z0 + h2z);
		elseif (shape == 43)
			% Z-pyramid
			z1 = pos(1);
			z2 = pos(2);
			x0 = pos(3);
			y0 = pos(4);
			h1x = pos(5) / 2;
			h1y = pos(6) / 2;
			h2x = pos(7) / 2;
			h2y = pos(8) / 2;
			[g, p, n] = rectangle_data(g, p, n, type, 'Z', z1, x0 - h1x, y0 - h1y, x0 + h1x, y0 + h1y);
			[g, p, n] = rectangle_data(g, p, n, type, 'Z', z2, x0 - h2x, y0 - h2y, x0 + h2x, y0 + h2y);
			[g, p, n] = line_data(g, p, n, type, x0 - h1x, y0 - h1y, z1, x0 - h2x, y0 - h2y, z2);
			[g, p, n] = line_data(g, p, n, type, x0 - h1x, y0 + h1y, z1, x0 - h2x, y0 + h2y, z2);
			[g, p, n] = line_data(g, p, n, type, x0 + h1x, y0 - h1y, z1, x0 + h2x, y0 - h2y, z2);
			[g, p, n] = line_data(g, p, n, type, x0 + h1x, y0 + h1y, z1, x0 + h2x, y0 + h2y, z2);
		elseif (shape == 51)
			% X-cone
			x1 = pos(1);
			x2 = pos(2);
			y0 = pos(3);
			z0 = pos(4);
			r1y = pos(5) / 2;
			r1z = pos(6) / 2;
			r2y = pos(7) / 2;
			r2z = pos(8) / 2;
			[g, p, n] = ellipse_data(g, p, n, type, 'X', x1, y0 - r1y, z0 - r1z, y0 + r1y, z0 + r1z, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'X', x2, y0 - r2y, z0 - r2z, y0 + r2y, z0 + r2z, nc);
			[g, p, n] = line_data(g, p, n, type, x1, y0,       z0 + r1z, x2, y0,       z0 + r2z);
			[g, p, n] = line_data(g, p, n, type, x1, y0,       z0 - r1z, x2, y0,       z0 - r2z);
			[g, p, n] = line_data(g, p, n, type, x1, y0 + r1y, z0,       x2, y0 + r2y, z0      );
			[g, p, n] = line_data(g, p, n, type, x1, y0 - r1y, z0,       x2, y0 - r2y, z0      );
		elseif (shape == 52)
			% Y-cone
			y1 = pos(1);
			y2 = pos(2);
			z0 = pos(3);
			x0 = pos(4);
			r1z = pos(5) / 2;
			r1x = pos(6) / 2;
			r2z = pos(7) / 2;
			r2x = pos(8) / 2;
			[g, p, n] = ellipse_data(g, p, n, type, 'Y', y1, z0 - r1z, x0 - r1x, z0 + r1z, x0 + r1x, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'Y', y2, z0 - r2z, x0 - r2x, z0 + r2z, x0 + r2x, nc);
			[g, p, n] = line_data(g, p, n, type, x0,       y1, z0 + r1z, x0,       y2, z0 + r2z);
			[g, p, n] = line_data(g, p, n, type, x0,       y1, z0 - r1z, x0,       y2, z0 - r2z);
			[g, p, n] = line_data(g, p, n, type, x0 + r1x, y1, z0,       x0 + r2x, y2, z0      );
			[g, p, n] = line_data(g, p, n, type, x0 - r1x, y1, z0,       x0 - r2x, y2, z0      );
		elseif (shape == 53)
			% Z-cone
			z1 = pos(1);
			z2 = pos(2);
			x0 = pos(3);
			y0 = pos(4);
			r1x = pos(5) / 2;
			r1y = pos(6) / 2;
			r2x = pos(7) / 2;
			r2y = pos(8) / 2;
			[g, p, n] = ellipse_data(g, p, n, type, 'Z', z1, x0 - r1x, y0 - r1y, x0 + r1x, y0 + r1y, nc);
			[g, p, n] = ellipse_data(g, p, n, type, 'Z', z2, x0 - r2x, y0 - r2y, x0 + r2x, y0 + r2y, nc);
			[g, p, n] = line_data(g, p, n, type, x0,       y0 + r1y, z1, x0,       y0 + r2y, z2);
			[g, p, n] = line_data(g, p, n, type, x0,       y0 - r1y, z1, x0,       y0 - r2y, z2);
			[g, p, n] = line_data(g, p, n, type, x0 + r1x, y0,       z1, x0 + r2x, y0,       z2);
			[g, p, n] = line_data(g, p, n, type, x0 - r1x, y0,       z1, x0 - r2x, y0,       z2);
		end
	end
end

% line data
function [g, p, n] = line_data(g, p, n, type, x1, y1, z1, x2, y2, z2)
	n = n + 1;
	g(n, 1, 1:3) = [x1 y1 z1];
	g(n, 2, 1:3) = [x2 y2 z2];
	p(n) = type;
end

% rectangle data
function [g, p, n] = rectangle_data(g, p, n, type, dir, c0, p1, q1, p2, q2)
	if     (dir == 'X')
		[g, p, n] = line_data(g, p, n, type, c0, p1, q1, c0, p2, q1);
		[g, p, n] = line_data(g, p, n, type, c0, p2, q1, c0, p2, q2);
		[g, p, n] = line_data(g, p, n, type, c0, p2, q2, c0, p1, q2);
		[g, p, n] = line_data(g, p, n, type, c0, p1, q2, c0, p1, q1);
	elseif (dir == 'Y')
		[g, p, n] = line_data(g, p, n, type, q1, c0, p1, q1, c0, p2);
		[g, p, n] = line_data(g, p, n, type, q1, c0, p2, q2, c0, p2);
		[g, p, n] = line_data(g, p, n, type, q2, c0, p2, q2, c0, p1);
		[g, p, n] = line_data(g, p, n, type, q2, c0, p1, q1, c0, p1);
	elseif (dir == 'Z')
		[g, p, n] = line_data(g, p, n, type, p1, q1, c0, p2, q1, c0);
		[g, p, n] = line_data(g, p, n, type, p2, q1, c0, p2, q2, c0);
		[g, p, n] = line_data(g, p, n, type, p2, q2, c0, p1, q2, c0);
		[g, p, n] = line_data(g, p, n, type, p1, q2, c0, p1, q1, c0);
	end
end

% ellipse data
function [g, p, n] = ellipse_data(g, p, n, type, dir, c0, p1, q1, p2, q2, nc)
	p0 = (p1 + p2) / 2;
	q0 = (q1 + q2) / 2;
	pr = abs(p1 - p2) / 2;
	qr = abs(q1 - q2) / 2;
	for i = 1 : nc
		a1 = (i - 1) * (2 * pi) / nc;
		a2 = (i - 0) * (2 * pi) / nc;
		if     (dir == 'X')
			x1 = c0;
			y1 = p0 + pr * cos(a1);
			z1 = q0 + qr * sin(a1);
			x2 = c0;
			y2 = p0 + pr * cos(a2);
			z2 = q0 + qr * sin(a2);
		elseif (dir == 'Y')
			y1 = c0;
			z1 = p0 + pr * cos(a1);
			x1 = q0 + qr * sin(a1);
			y2 = c0;
			z2 = p0 + pr * cos(a2);
			x2 = q0 + qr * sin(a2);
		elseif (dir == 'Z')
			z1 = c0;
			x1 = p0 + pr * cos(a1);
			y1 = q0 + qr * sin(a1);
			z2 = c0;
			x2 = p0 + pr * cos(a2);
			y2 = q0 + qr * sin(a2);
		end
		[g, p, n] = line_data(g, p, n, type, x1, y1, z1, x2, y2, z2);
	end
end

% No. of lines
function nline = alloc_lines(Geom, nc)
	nline = 0;
	for n = 1 : length(Geom)
		shape = Geom(n).shape;
		if (shape == 1) || (shape == 41) || (shape == 42) || (shape == 43)
			nl = 12;
		elseif (shape == 2)
			nl = 3 * nc;
		elseif (shape == 11) || (shape == 12) || (shape == 13) || (shape == 51) || (shape == 52) || (shape == 53)
			nl = 2 * nc + 4;
		elseif (shape == 31) || (shape == 32) || (shape == 33)
			nl = 9;
		end
		nline = nline + nl;
	end
end
