% in geometry ?
function b = ingeometry(x, y, z, shape, g)
	b = false;
	eps = 1e-8;
	eps2 = eps^2;

	% rectangle
	if     (shape == 1)
		b = ((x - g(1)) * (x - g(2)) <= eps2) && ...
		    ((y - g(3)) * (y - g(4)) <= eps2) && ...
		    ((z - g(5)) * (z - g(6)) <= eps2);
	% ellipsoid
	elseif (shape == 2)
		x0 = (g(1) + g(2)) / 2;
		y0 = (g(3) + g(4)) / 2;
		z0 = (g(5) + g(6)) / 2;
		xr = abs(g(1) - g(2)) / 2;
		yr = abs(g(3) - g(4)) / 2;
		zr = abs(g(5) - g(6)) / 2;
		b = ((x - x0) / xr)^2 + ((y - y0) / yr)^2 + ((z - z0) / zr)^2 < (1 + eps);
	% cylinder
	elseif (shape == 11)
		% X cylinder
		y0 = (g(3) + g(4)) / 2;
		z0 = (g(5) + g(6)) / 2;
		yr = abs(g(3) - g(4)) / 2;
		zr = abs(g(5) - g(6)) / 2;
		b = ((x - g(1)) * (x - g(2)) < eps2) && ...
		    ((y - y0) / yr)^2 + ((z - z0) / zr)^2 < (1 + eps);
	elseif (shape == 12)
		% Y clinder
		x0 = (g(1) + g(2)) / 2;
		z0 = (g(5) + g(6)) / 2;
		xr = abs(g(1) - g(2)) / 2;
		zr = abs(g(5) - g(6)) / 2;
		b = ((y - g(3)) * (y - g(4)) < eps2) && ...
		    ((z - z0) / zr)^2 + ((x - x0) / xr)^2 < (1 + eps);
	elseif (shape == 13)
		% Z cylinder
		x0 = (g(1) + g(2)) / 2;
		y0 = (g(3) + g(4)) / 2;
		xr = abs(g(1) - g(2)) / 2;
		yr = abs(g(3) - g(4)) / 2;
		b = ((z - g(5)) * (z - g(6)) < eps2) && ...
		    ((x - x0) / xr)^2 + ((y - y0) / yr)^2 < (1 + eps);
	% pillar
	elseif (shape == 31)
		% X-pillar
		if ((x - g(1)) * (x - g(2)) > eps2)
			return;
		end
		tri = [g(3) g(6); g(4) g(7); g(5) g(8)];
		b = inout3(y, z, tri);
	elseif (shape == 32)
		% Y-pillar
		if ((y - g(1)) * (y - g(2)) > eps2)
			return;
		end
		tri = [g(3) g(6); g(4) g(7); g(5) g(8)];
		b = inout3(z, x, tri);
	elseif (shape == 33)
		% Z-pillar
		if ((z - g(1)) * (z - g(2)) > eps2)
			return;
		end
		tri = [g(3) g(6); g(4) g(7); g(5) g(8)];
		b = inout3(x, y, tri);
	% pyramid
	elseif (shape == 41)
		% X-pyramid
		x1  = g(1);
		x2  = g(2);
		y0  = g(3);
		z0  = g(4);
		h1y = g(5) / 2;
		h1z = g(6) / 2;
		h2y = g(7) / 2;
		h2z = g(8) / 2;
		if (abs(x1 - x2) > eps)
			f = (x - x1) / (x2 - x1);
		else
			f = 0.5;
		end
		hy = h1y + f * (h2y - h1y);
		hz = h1z + f * (h2z - h1z);
		b = ((x - x1) * (x - x2) < eps2) && ...
		    (abs(y - y0) <= hy) && ...
		    (abs(z - z0) <= hz);
	elseif (shape == 42)
		% Y-pyramid
		y1  = g(1);
		y2  = g(2);
		z0  = g(3);
		x0  = g(4);
		h1z = g(5) / 2;
		h1x = g(6) / 2;
		h2z = g(7) / 2;
		h2x = g(8) / 2;
		if (abs(y1 - y2) > eps)
			f = (y - y1) / (y2 - y1);
		else
			f = 0.5;
		end
		hz = h1z + f * (h2z - h1z);
		hx = h1x + f * (h2x - h1x);
		b = ((y - y1) * (y - y2) < eps2) && ...
		    (abs(z - z0) <= hz) && ...
		    (abs(x - x0) <= hx);
	elseif (shape == 43)
		% Z-pyramid
		z1  = g(1);
		z2  = g(2);
		x0  = g(3);
		y0  = g(4);
		h1x = g(5) / 2;
		h1y = g(6) / 2;
		h2x = g(7) / 2;
		h2y = g(8) / 2;
		if (abs(z1 - z2) > eps)
			f = (z - z1) / (z2 - z1);
		else
			f = 0.5;
		end
		hx = h1x + f * (h2x - h1x);
		hy = h1y + f * (h2y - h1y);
		b = ((z - z1) * (z - z2) < eps2) && ...
		    (abs(x - x0) <= hx) && ...
		    (abs(y - y0) <= hy);
	% cone
	elseif (shape == 51)
		% X-cone
		x1  = g(1);
		x2  = g(2);
		y0  = g(3);
		z0  = g(4);
		r1y = g(5) / 2;
		r1z = g(6) / 2;
		r2y = g(7) / 2;
		r2z = g(8) / 2;
		if (abs(x1 - x2) > eps)
			f = (x - x1) / (x2 - x1);
		else
			f = 0.5;
		end
		ry = r1y + f * (r2y - r1y);
		rz = r1z + f * (r2z - r1z);
		b = ((x - x1) * (x - x2) < eps2) && ...
		    (((y - y0) / ry)^2 + ((z - z0) / rz)^2 <= 1);
	elseif (shape == 52)
		% Y-cone
		y1  = g(1);
		y2  = g(2);
		z0  = g(3);
		x0  = g(4);
		r1z = g(5) / 2;
		r1x = g(6) / 2;
		r2z = g(7) / 2;
		r2x = g(8) / 2;
		if (abs(y1 - y2) > eps)
			f = (y - y1) / (y2 - y1);
		else
			f = 0.5;
		end
		rz = r1z + f * (r2z - r1z);
		rx = r1x + f * (r2x - r1x);
		b = ((y - y1) * (y - y2) < eps2) && ...
		    (((z - z0) / rz)^2 + ((x - x0) / rx)^2 <= 1);
	elseif (shape == 53)
		% Z-cone
		z1  = g(1);
		z2  = g(2);
		x0  = g(3);
		y0  = g(4);
		r1x = g(5) / 2;
		r1y = g(6) / 2;
		r2x = g(7) / 2;
		r2y = g(8) / 2;
		if (abs(z1 - z2) > eps)
			f = (z - z1) / (z2 - z1);
		else
			f = 0.5;
		end
		rx = r1x + f * (r2x - r1x);
		ry = r1y + f * (r2y - r1y);
		b = ((z - z1) * (z - z2) < eps2) && ...
		    (((x - x0) / rx)^2 + ((y - y0) / ry)^2 <= 1);
	end
end

% 1 : in or border, 0 : out
function ret = inout3(x, y, tri)
	ret = false;
	eps = 1e-8;

	xmin = min(tri(:, 1));
	xmax = max(tri(:, 1));
	ymin = min(tri(:, 2));
	ymax = max(tri(:, 2));
	zero = eps * (abs(xmax - xmin) + abs(ymax - ymin));

	x1 = tri(1, 1);
	x2 = tri(2, 1);
	x3 = tri(3, 1);
	y1 = tri(1, 2);
	y2 = tri(2, 2);
	y3 = tri(3, 2);

	det = ((x2 - x1) * (y3 - y1)) - ((x3 - x1) * (y2 - y1));

	if (abs(det) > zero^2)
		a = + ((x - x1) * (y3 - y1) - (y - y1) * (x3 - x1)) / det;
		b = - ((x - x1) * (y2 - y1) - (y - y1) * (x2 - x1)) / det;
		if (a > -eps) && (b > -eps) && (a + b < 1 + eps)
			ret = true;
		end
	end
end
