% monitor
function monitor(varargin)
	if (nargin < 2)
		return
	end

	key = varargin{1};
	fid = varargin{2};

	switch key
		case 1
			if (nargin > 2)
				monitor1(fid, varargin{3});
			end
		case 2
			if (nargin > 4)
				monitor2(fid, varargin{3}, varargin{4}, varargin{5});
			end
		case 3
			if (nargin > 4)
				monitor3(fid, varargin{3}, varargin{4}, varargin{5});
			end
		case 4
			monitor4(fid);
		case 5
			if (nargin > 2)
				monitor5(fid, varargin{3});
			end
		otherwise
	end
end

function monitor1(fid, msg)
	monitor1_(fid, msg);
	monitor1_(1,   msg);
end

function monitor2(fid, Parm, Mesh, Geom)
	monitor2_(fid, Parm, Mesh, Geom);
	monitor2_(1,   Parm, Mesh, Geom);
end

function monitor3(fid, Parm, charge, wtotal)
	monitor3_(fid, Parm, charge, wtotal);
	monitor3_(1,   Parm, charge, wtotal);
end

function monitor4(fid)
	monitor4_(fid);
	monitor4_(1);
end

function monitor5(fid, cpu)
	monitor5_(fid, cpu);
	monitor5_(1,   cpu);
end

function monitor1_(fid, msg)
	fprintf(fid, '%s\n', msg);
end

function monitor2_(fid, Parm, Mesh, Geom)
	nxc = Mesh.nx - 1;
	nyc = Mesh.ny - 1;
	nzc = Mesh.nz - 1;

	nvolt = 0;
	nepsr = 0;
	for n = 1 : length(Geom)
		if     (Geom(n).type == 1)
			nvolt = nvolt + 1;
		elseif (Geom(n).type == 2)
			nepsr = nepsr + 1;
		end
	end

	fprintf(fid, '%s\n', datetime);  % MATLAB
	%fprintf(fid, '%s\n', datestr(now));  % Octave
	fprintf(fid, 'Title = %s\n', Parm.title);
	fprintf(fid, 'Cells = %d x %d x %d = %d\n', nxc, nyc, nzc, nxc * nyc * nzc);
	fprintf(fid, 'No. of voltages    = %d\n', length(Parm.volt));
	fprintf(fid, 'No. of dielecrics  = %d\n', length(Parm.epsr));
	fprintf(fid, 'No. of geometries  = %d + %d = %d\n', nvolt, nepsr, length(Geom));
	fprintf(fid, 'Memory size        = %d [MB]\n', memory_size(Mesh));
	fprintf(fid, 'Omega              = %g\n', Parm.solver(1));
	fprintf(fid, 'Max iterations     = %d\n', Parm.solver(2));
	fprintf(fid, 'Convergence = %.7f\n', Parm.solver(4));
	fprintf(fid, '=== iteration start ===\n');
	fprintf(fid, '      step    residual\n');
end

function monitor3_(fid, Parm, charge, wtotal)
	fprintf(fid, '%s\n', 'Volt#   Q[C]');
	for n = 1 : length(Parm.volt)
		fprintf(fid, '%5d %13.5e\n', n, charge(n));
	end
	fprintf(fid, 'Total energy [W] = %.5e\n', wtotal);
end

function monitor4_(fid)
	fprintf(fid, '=== output files ===\n');
	fprintf(fid, 'ost.log, ost.mat\n');
end

function monitor5_(fid, cpu)
	fprintf(fid, '%s\n', '=== normal end ===');
	fprintf(fid, '%s\n', datetime);  % MATLAB
	%fprintf(fid, '%s\n', datestr(now));  % Octave
	fprintf(fid, '%s\n', '=== cpu time [sec] ===');
	fprintf(fid, 'part-1 : %10.3f\n', cpu(1));
	fprintf(fid, 'part-2 : %10.3f\n', cpu(2));
	fprintf(fid, 'part-3 : %10.3f\n', cpu(3));
	fprintf(fid, '%s\n', '-------------------');
	fprintf(fid, 'total  : %10.3f\n', sum(cpu));
end

% memory size [MB]
function mem = memory_size(Mesh)
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;
	n = nx * ny * nz;
	size = 3 * n * 8;   % V, idVolt, Epsr
	mem = round(size / 1024 / 1024) + 1;
end
