% plot V/E/D/Q along lines (2D)
function plot1d(Parm, Mesh, Post, idVolt, Epsr, V)
	% component, unit, color
	icompolist = ["V" "E" "Ex" "Ey" "Ez" "D" "Dx" "Dy" "Dz" "Q"];
	compolist = ["V" "E" "D" "Q"];
	unitlist = ["[V]" "[V/m]" "[C/m^2]" "[C/m^3]"; "[V]" "[dBV/m]" "[dBC/m^2]" "[dBC/m^3]"];
	lcolor = ['k' 'k' 'r' 'g' 'b' 'k' 'r' 'g' 'b' 'k'];

	% 1d.log
	if (Post.p1dlog == 1)
		fname = '1d.log';
		[fid, errmsg] = fopen(fname, 'wt');
		if (fid < 0)
			fprintf('%s : %s\n', errmsg, fname);
			return;
		end
	end

	% mesh
	xn = Mesh.xn;
	yn = Mesh.yn;
	zn = Mesh.zn;
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;

	for il = 1 : length(Post.p1ddir)
		compo = Post.p1dcompo(il);
		icompo = find(startsWith(icompolist, compo));  % 1, [2 3 4 5], [6 7 8 9], 10

		% X, Y data
		pos = Post.p1dpos(il, :);
		if     (Post.p1ddir(il) == "X")
			pos(1) = max(yn(1), min(yn(end), pos(1)));
			pos(2) = max(zn(1), min(zn(end), pos(2)));
			[~, j] = min(abs(pos(1) - yn));
			[~, k] = min(abs(pos(2) - zn));
			x = xn;
			y = zeros(nx, 10);
			for i = 1 : nx
				y(i, :) = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k);
			end
			strline = sprintf('Y = %g [m], Z = %g [m]', pos(1), pos(2));
		elseif (Post.p1ddir(il) == "Y")
			pos(1) = max(zn(1), min(zn(end), pos(1)));
			pos(2) = max(xn(1), min(xn(end), pos(2)));
			[~, k] = min(abs(pos(1) - zn));
			[~, i] = min(abs(pos(2) - xn));
			x = yn;
			y = zeros(ny, 10);
			for j = 1 : ny
				y(j, :) = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k);
			end
			strline = sprintf('Z = %g [m], X = %g [m]', pos(1), pos(2));
		elseif (Post.p1ddir(il) == "Z")
			pos(1) = max(xn(1), min(xn(end), pos(1)));
			pos(2) = max(yn(1), min(yn(end), pos(2)));
			[~, i] = min(abs(pos(1) - xn));
			[~, j] = min(abs(pos(2) - yn));
			x = zn;
			y = zeros(nz, 10);
			for k = 1 : nz
				y(k, :) = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k);
			end
			strline = sprintf('X = %g [m], Y = %g [m]', pos(1), pos(2));
		end

		% to dB (exclude V)
		if (Post.p1ddb == 1)
			y(:, 2:end) = 20 * log10(max(abs(y(:, 2:end)), 1e-5));
		end

		% log
		if (Post.p1dlog == 1) && (fid > 0)
			log_1d(fid, Post, il, x, y);
		end

		% Y-limit
		ymin = min(min(y(:, icompo)));
		ymax = max(max(y(:, icompo)));
		if (Post.p1dscale(1) == 1)
			y(:, icompo) = max(Post.p1dscale(2), min(y(:, icompo), Post.p1dscale(3)));
		end

		% figure
		strfig = sprintf('OpenSTF - %s along %s-line (%d/%d)', compo, Post.p1ddir(il), il, length(Post.p1ddir));
		f = figure('Name', strfig, 'NumberTitle', 'off');
		f.Position = Post.rsc2d.pos;
	
		% plot (1 or 4 lines)
		ic1 = icompo(1);
		ic2 = icompo(end);
		for ic = ic1 : ic2
			p = line(x, y(:, ic));
			p.Color = lcolor(ic);
			if (Post.marker > 1)
				p.Marker = 'o';
				p.MarkerSize = Post.marker;
			end
			%hold on;
		end
		grid on;

		% X-axis
		xlim([x(1) x(end)]);
	
		% Y-axis (user scale)
		if (Post.p1dscale(1) == 1)
			ylim([Post.p1dscale(2) Post.p1dscale(3)]);
			ydiv = linspace(Post.p1dscale(2), Post.p1dscale(3), Post.p1dscale(4) + 1);
			yticks(ydiv);
		end
	
		% legend
		if (length(icompo) > 1)
			legend(compo, compo + "x", compo + "y", compo + "z");
		end
	
		% component
		strunit = unitlist(Post.p1ddb + 1, (compolist == compo));
		strcompo = compo + " " + strunit;

		% label
		xlabel(sprintf('%s [m]', Post.p1ddir(il)));
		ylabel(strcompo);
	
		% V=0 line
		if (compo == "V") && (ymin < 0) && (ymax > 0)
			p = line([x(1) x(end)], [0 0]);
			p.Color = 'b';
		end
		
		% title
		strmax = sprintf('max = %.4g %s, min = %.4g %s', ymax, strunit, ymin, strunit);
		title(sprintf('%s\n%s, %s, %s', Parm.title, strcompo, strline, strmax));
				
		% font
		fontsize(gcf, Post.rsc2d.fontsize, 'pixels');
		fontname(gcf, Post.rsc2d.fontname);
	end
end

% log
function log_1d(fid, Post, il, x, y)
	% header
	fprintf(fid, '   No.          %c[m]          V[V]', Post.p1ddir(il));	
	if (Post.p1ddb == 0)
		fprintf(fid, '        E[V/m]       Ex[V/m]       Ey[V/m]       Ez[V/m]      D[C/m^2]     Dx[C/m^2]     Dy[C/m^2]     Dz[C/m^2]      Q[C/m^3]\n');
		fmt = '%14.5e';
	else
		fprintf(fid, '     E[dBV/m]    Ex[dBV/m]    Ey[dBV/m]    Ez[dBV/m]   D[dBC/m^2]  Dx[dBC/m^2]  Dy[dBC/m^2]  Dz[dBC/m^2]   Q[dBC/m^3]\n');
		fmt = '%13.5f';
	end

	% data
	for n = 1 : size(y, 1)
		fprintf(fid, '%6d%14.5e%14.5e', n, x(n), y(n, 1));
		for ic = 2 : 10
			fprintf(fid, fmt, y(n, ic));
		end
		fprintf(fid, '\n');
	end
end
