% plot V/E/D/Q on planes (2D)
function plot2d(Parm, Mesh, Geom, Post, idVolt, Epsr, V)
	% component and unit
	compolist = ["V" "E" "Ex" "Ey" "Ez" "D" "Dx" "Dy" "Dz" "Q"];
	unitlist = ["[V]" "[V/m]" "[V/m]" "[V/m]" "[V/m]" "[C/m^2]" "[C/m^2]" "[C/m^2]" "[C/m^2]" "[C/m^3]";
		"[V]" "[dBV/m]" "[dBV/m]" "[dBV/m]" "[dBV/m]" "[dBC/m^2]" "[dBC/m^2]" "[dBC/m^2]" "[dBC/m^2]" "[dBC/m^3]"];

	% 2d.log
	if (Post.p2dlog == 1)
		fname = '2d.log';
		[fid, errmsg] = fopen(fname, 'wt');
		if (fid < 0)
			fprintf('%s : %s\n', errmsg, fname);
			return;
		end
	end
	
	% make geometry lines for plot
	if (Post.p2dobject(1) == 1)
		[gline, pline, nline] = geomlines(Geom);
	end

	% mesh
	xn = Mesh.xn;
	yn = Mesh.yn;
	zn = Mesh.zn;
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;

	for ip = 1 : length(Post.p2ddir)
		compo = Post.p2dcompo(ip);
		icompo = (compolist == compo);  % logical array

		% X, Y data
		pos = Post.p2dpos(ip);
		if     (Post.p2ddir(ip) == "X")
			pos = max(xn(1), min(xn(end), pos));
			[~, i] = min(abs(pos - xn));
			x = yn;
			y = zn;
			z = zeros(nz, ny);
			zall = zeros(nz, ny, 10);
			for j = 1 : ny
			for k = 1 : nz
				zall(k, j, :) = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k);
				z(k, j) = zall(k, j, icompo);
			end
			end
			xstr = "Y [m]";
			ystr = "Z [m]";
			strplane = sprintf('X = %g [m]', pos);
		elseif (Post.p2ddir(ip) == "Y")
			pos = max(yn(1), min(yn(end), pos));
			[~, j] = min(abs(pos - yn));
			x = xn;
			y = zn;
			z = zeros(nz, nx);
			zall = zeros(nz, nx, 10);
			for i = 1 : nx
			for k = 1 : nz
				zall(k, i, :) = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k);
				z(k, i) = zall(k, i, icompo);
			end
			end
			xstr = "X [m]";
			ystr = "Z [m]";
			strplane = sprintf('Y = %g [m]', pos);
		elseif (Post.p2ddir(ip) == "Z")
			pos = max(zn(1), min(zn(end), pos));
			[~, k] = min(abs(pos - zn));
			x = xn;
			y = yn;
			z = zeros(ny, nx);
			zall = zeros(ny, nx, 10);
			for i = 1 : nx
			for j = 1 : ny
				zall(j, i, :) = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k);
				z(j, i) = zall(j, i, icompo);
			end
			end
			xstr = "X [m]";
			ystr = "Y [m]";
			strplane = sprintf('Z = %g [m]', pos);
		end

		% to dB
		if (Post.p2ddb == 1)
			if (compo ~= "V")
				z(:) = 20 * log10(max(abs(z(:)), 1e-5));
			end
			zall(:, 2:end) = 20 * log10(max(abs(zall(:, 2:end)), 1e-5));
		end

		% log
		if (Post.p2dlog == 1) && (fid > 0)
			log_2d(fid, Post, ip, x, y, zall, xstr, ystr);
		end

		% Z-limit
		zmin = min(min(z(:)));
		zmax = max(max(z(:)));
		if (zmin == zmax)
			disp("*** plot2d : min = max");
			return
		end
		if (Post.p2dscale(1) == 1)
			cmin = Post.p2dscale(2);
			cmax = Post.p2dscale(3);
		else
			cmin = zmin;
			cmax = zmax;
		end
		z(:) = max(cmin, min(z(:), cmax));

		% figure
		strfig = sprintf('OpenSTF - %s on %s-plane (%d/%d)', compo, Post.p2ddir(ip), ip, length(Post.p2ddir));
		f = figure('Name', strfig, 'NumberTitle', 'off');
		f.Position = Post.rsc2d.pos;
		f.Position(1) = Post.rsc2d.pos(1) + (-(length(Post.p2ddir) + 1) / 2 + ip) * 40;  % shift

		% plot
		if     (Post.p2dcontour == 0) || (Post.p2dcontour == 2)
			contourf(x, y, z, Post.p2dscale(4));
		elseif (Post.p2dcontour == 1) || (Post.p2dcontour == 3)
			contour(x, y, z, Post.p2dscale(4));
		end
		axis equal;

		% zoom
		if (Post.p2dzoom(1) == 1)
			xlim([Post.p2dzoom(2) Post.p2dzoom(3)]);
			ylim([Post.p2dzoom(4) Post.p2dzoom(5)]);
		end

		% colormap
		if     (Post.p2dcontour == 0) || (Post.p2dcontour == 1)
			%colormap parula;
			colormap jet;
		elseif (Post.p2dcontour == 2) || (Post.p2dcontour == 3)
			colormap gray;
		end

		% component
		strunit = unitlist(Post.p2ddb + 1, icompo);
		strcompo = compo + " " + strunit;

		% colorbar
		cbar = colorbar();
		cbar.Label.String = strcompo;
		cbar.Limits = [cmin cmax];

		% label
		xlabel(xstr);
		ylabel(ystr);

		% title
		strmax = sprintf('max = %.5g %s, min = %.5g %s', zmax, strunit, zmin, strunit);
		title(sprintf('%s\n%s, %s, %s', Parm.title, strcompo, strplane, strmax));

		% plot geometry lines
		if (Post.p2dobject(1) == 1)
			plot2d_geom(gline, pline, nline, Post.p2ddir(ip), Post.p2dobject(2:3));
		end
			
		% font
		fontsize(gcf, Post.rsc2d.fontsize, 'pixels');
		fontname(gcf, Post.rsc2d.fontname);
	end
end

% plot geometry lines
function plot2d_geom(gline, pline, nline, dir, type)
	for n = 1 : nline
		if     (dir == "X")
			m1 = 2;
			m2 = 3;
		elseif (dir == "Y")
			m1 = 1;
			m2 = 3;
		elseif (dir == "Z")
			m1 = 1;
			m2 = 2;
		end

		% electrodes
		if (type(1) == 1) && (pline(n) == 1)
			p = line([gline(n, 1, m1) gline(n, 2, m1)], [gline(n, 1, m2) gline(n, 2, m2)]);
			p.Color = 'k';
		end

		% dielectrics
		if (type(2) == 1) && (pline(n) == 2)
			p = line([gline(n, 1, m1) gline(n, 2, m1)], [gline(n, 1, m2) gline(n, 2, m2)]);
			p.Color = 'm';
		end
	end
end

% log
function log_2d(fid, Post, ip, x, y, z, xstr, ystr)
	% header
	fprintf(fid, '   No.   No.         %s         %s          V[V]', xstr, ystr);
	if (Post.p2ddb == 0)
		fprintf(fid, '        E[V/m]       Ex[V/m]       Ey[V/m]       Ez[V/m]      D[C/m^2]     Dx[C/m^2]     Dy[C/m^2]     Dz[C/m^2]      Q[C/m^3]\n');
		fmt = '%14.5e';
	else
		fprintf(fid, '     E[dBV/m]    Ex[dBV/m]    Ey[dBV/m]    Ez[dBV/m]   D[dBC/m^2]  Dx[dBC/m^2]  Dy[dBC/m^2]  Dz[dBC/m^2]   Q[dBC/m^3]\n');
		fmt = '%13.5f';
	end

	% data
	for n1 = 1 : length(x)
	for n2 = 1 : length(y)
		fprintf(fid, "%6d%6d%14.5e%14.5e", n1, n2, x(n1), y(n2));
		fprintf(fid, "%14.5e", z(n2, n1, 1));
		for ic = 2 : 10
			fprintf(fid, fmt, z(n2, n1, ic));
		end
		fprintf(fid, '\n');
	end
	end
end
