% plot geometry (3D)
function plot3d_geom(Parm, Mesh, Geom)
	% mesh
	xn = Mesh.xn;
	yn = Mesh.yn;
	zn = Mesh.zn;
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;

	nline = nx + ny + nz;
	x = zeros(3, nline);
	y = zeros(3, nline);
	z = zeros(3, nline);

	n = 0;
	% X-const
	for i = 1 : nx
		n = n + 1;
		x(:, n) = [xn(i)   xn(i) xn(i)  ];
		y(:, n) = [yn(1)   yn(1) yn(end)];
		z(:, n) = [zn(end) zn(1) zn(1)  ];
	end
	% Y-const
	for j = 1 : ny
		n = n + 1;
		x(:, n) = [xn(end) xn(1) xn(1)  ];
		y(:, n) = [yn(j)   yn(j) yn(j)  ];
		z(:, n) = [zn(1)   zn(1) zn(end)];
	end
	% Z-const
	for k = 1 : nz
		n = n + 1;
		x(:, n) = [xn(1)   xn(1) xn(end)];
		y(:, n) = [yn(end) yn(1) yn(1)  ];
		z(:, n) = [zn(k)   zn(k) zn(k)  ];
	end

	% figure
	f = figure('Name', "OpenSTF - geometry (3D)", 'NumberTitle', 'off');
    f.Position = [100, 100, 500, 500];

	% plot mesh
	col = [0.7 0.7 0.7];
	plot3(x, y, z, Color=col);
	hold on;

	% plot geometry
	[gline, pline, nline] = geomlines(Geom);
	for n = 1 : nline
		p = line([gline(n, 1, 1) gline(n, 2, 1)], [gline(n, 1, 2) gline(n, 2, 2)], [gline(n, 1, 3) gline(n, 2, 3)]);
		if     (pline(n) == 1)
			p.Color = 'k';
		elseif (pline(n) == 2)
			p.Color = 'm';
		end
	end

	axis equal;
	%light;
	view(120, 30);

	% label
	xlabel('X[m]');
	ylabel('Y[m]');
	zlabel('Z[m]');

	% title
	nvolt = 0;
	nepsr = 0;
	for n = 1 : length(Geom)
		nvolt = nvolt + (Geom(n).type == 1);
		nepsr = nepsr + (Geom(n).type == 2);
	end
	title(sprintf('%s\nelectrodes = %d, dielectrics = %d\ncells = %d x %d x %d = %d', ...
		Parm.title, nvolt, nepsr, nx - 1, ny - 1, nz - 1, (nx - 1) * (ny - 1) * (nz - 1)));

	% XYZ limit
	gx = [xn(1) xn(end)];
	gy = [yn(1) yn(end)];
	gz = [zn(1) zn(end)];
	margin = 0.02 * (abs(gx(2)- gx(1)) + abs(gy(2)- gy(1)) + abs(gz(2) - gz(1)));
	xlim([gx(1) - margin, gx(2) + margin]);
	ylim([gy(1) - margin, gy(2) + margin]);
	zlim([gz(1) - margin, gz(2) + margin]);

	% font
	fontsize(gcf, 13, 'pixels');
	%fontname(gcf, 'Monospaced');
end
