% read post data
function Post = post_data(fid)
	% component
	compolist = ["V" "E" "Ex" "Ey" "Ez" "D" "Dx" "Dy" "Dz" "Q"];
	dirlist = ["X" "Y" "Z"];

	% initialize
	Post.iter = 0;
	Post.p1dcompo = "";
	Post.p1ddir = "";
	Post.p1dpos = zeros(1, 2);
	Post.p1ddb = 0;
	Post.p1dscale = [0 0 0 0];
	Post.p1dlog = 1;
	Post.p2dcompo = "";
	Post.p2ddir = "";
	Post.p2dpos = zeros(1, 1);
	%Post.p2dfigure = [1 0];  % not used
	Post.p2ddb = 0;
	Post.p2dscale = [0 0 0 20];  % (4) : new
	Post.p2dcontour = 0;
	Post.p2dobject = [1 1 1];
	Post.p2dzoom = [0 0 0 0 0];
	Post.p2dlog = 1;
	Post.marker = 0;  % new

	w2d = [0 0 0 0];
	%w3d = [0 0 0];  % not used

	n1d = 0;
	n2d = 0;

	% header
    tline = strtrim(fgetl(fid));
    d = split(tline);
    if (d{1} ~= "OpenSTF") || (length(d) < 3) 
        disp('Not OpenSTF data.');
        return
    end
    %version = (100 * str2double(d{2})) + str2double(d{3});

    % body
	while ~feof(fid)
        tline = strtrim(fgetl(fid));  %disp(tline);
        d = split(tline);  % cell array
        
        if (startsWith(d{1}, "#"))
            continue;
        elseif (strcmp(d{1}, "end"))
            break;
        elseif (length(d) < 3)
            continue;
        elseif (d{2} ~= "=")
            continue;
        end

        p = zeros(1, length(d) - 2);  % parameter array
        for i = 3 : length(d)
            p(i - 2) = str2double(d{i});
        end

		switch d{1}
			case "plotiter"
				Post.iter = p(1);
			case "plot1d"
				if (length(p) > 3) && any(ismember(compolist, d{3})) && any(ismember(dirlist, d{4}))
					n1d = n1d + 1;
					Post.p1dcompo(n1d) = d{3};  % string
					Post.p1ddir(n1d) = d{4};    % string
					Post.p1dpos(n1d, 1:2) = p(3:4);
				end
			case "1ddb"
				Post.p1ddb = p(1);
			case "1dscale"
				Post.p1dscale(1) = 1;
				if (length(p) > 2)
					%Post.p1dscale(2:4) = p(1:3);
					Post.p1dscale(2) = min(p(1:2));
					Post.p1dscale(3) = max(p(1:2));
					Post.p1dscale(4) = p(3);
				end
			case "1dlog"
				Post.p1dlog = p(1);
			case "plot2d"
				if (length(p) > 2) && any(ismember(compolist, d{3})) && any(ismember(dirlist, d{4}))
					n2d = n2d + 1;
					Post.p2dcompo(n2d) = d{3};  % string
					Post.p2ddir(n2d) = d{4};	% string
					Post.p2dpos(n2d, 1) = p(3);
				end
			%{
			case "2dfigure"
				if (length(p) > 1)
					Post.p2dfigure(1:2) = p(1:2);  % not used
				end
			%}
			case "2ddb"
				Post.p2ddb = p(1);
			case "2dscale"
				Post.p2dscale(1) = 1;
				if (length(p) > 2)
					%Post.p2dscale(2:4) = p(1:3);
					Post.p2dscale(2) = min(p(1:2));
					Post.p2dscale(3) = max(p(1:2));
					Post.p2dscale(4) = p(3);
				end
			case "2dcontour"
				Post.p2dcontour = p(1);
			case "2dobject"
				Post.p2dobject(1) = 1;
				if (length(p) > 1)
					Post.p2dobject(2:3) = p(1:2);
				end
			case "2dzoom"
				Post.p2dzoom(1) = 1;
				if (length(p) > 3)
					%Post.p2dzoom(2:5) = p(1:4);
					Post.p2dzoom(2) = min(p(1:2));
					Post.p2dzoom(3) = max(p(1:2));
					Post.p2dzoom(4) = min(p(3:4));
					Post.p2dzoom(5) = max(p(3:4));
				end
			case "2dlog"
				Post.p2dlog = p(1);
			case "marker"
				Post.marker = p(1);
			case 'window2d'
				if (length(p) > 3)
					w2d = [p(1) p(2) p(3) p(4)];
				end
			%{
			case 'window3d'
				if (length(p) > 5)
					w3d = [p(1) p(2) p(3) p(4) p(5) p(6)];  % not used
				end
			%}
			otherwise
		end
	end

	% === misc ===

	% resource
	scn = groot().ScreenSize;
	% 2D
	Post.rsc2d.pos = [scn(1) + (scn(3) - w2d(1)) / 2, scn(2) + (scn(4) - w2d(2)) / 2, w2d(1), w2d(2)];
	Post.rsc2d.fontsize = w2d(3);
	fnt = ["SansSerif", "Serif", "Monospaced"];
	Post.rsc2d.fontname = fnt(max(0, min(2, w2d(4))) + 1);

	%{
	% 3D
	Post.rsc3d.pos = [scn(1) + (scn(3) - w3d(1)) / 2, scn(2) + (scn(4) - w3d(2)) / 2, w3d(1), w3d(2)];
	Post.rsc3d.fontsize = w3d(3);
	%}
end
