% setup Volt index and Epsr array
function [idVolt, Epsr] = setup_id(Parm, Mesh, Geom)
	% mesh
	xn = Mesh.xn;
	yn = Mesh.yn;
	zn = Mesh.zn;
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;

	% alloc
	idVolt = zeros(nx, ny, nz);
	Epsr = ones(nx, ny, nz);  % = 1

	for n = 1 : length(Geom)
		type  = Geom(n).type;   % = 1/2
		pid   = Geom(n).pid;    % = 1,2,3,...
		shape = Geom(n).shape;  % = 1,2,11,...
		pos   = Geom(n).pos;    % position (6 or 8)
		if (shape == 1)
			% rectangle
			x1 = min(pos(1), pos(2));
			x2 = max(pos(1), pos(2));
			y1 = min(pos(3), pos(4));
			y2 = max(pos(3), pos(4));
			z1 = min(pos(5), pos(6));
			z2 = max(pos(5), pos(6));
			[i1, i2] = get_span(xn, x1, x2);
			[j1, j2] = get_span(yn, y1, y2);
			[k1, k2] = get_span(zn, z1, z2);
			for k = k1 : k2
			for j = j1 : j2
			for i = i1 : i2
				if     (type == 1)
					idVolt(i, j, k) = pid;
					Epsr(i, j, k) = 1;
				elseif (type == 2)
					idVolt(i, j, k) = 0;
					Epsr(i, j, k) = Parm.epsr(pid);
				end
			end
			end
			end
		else
			% not rectangle
			for k = 1 : nz
			for j = 1 : ny
			for i = 1 : nx
				if ingeometry(xn(i), yn(j), zn(k), shape, pos)
					if     (type == 1)
						idVolt(i, j, k) = pid;
						Epsr(i, j, k) = 1;
					elseif (type == 2)
						idVolt(i, j, k) = 0;
						Epsr(i, j, k) = Parm.epsr(pid);
					end
				end
			end
			end
			end
		end
	end
end

% p1 <= p(n1) <= p(n2) <= p2
function [n1, n2] = get_span(p, p1, p2)
	n1 = 1;
	n2 = length(p);
	b1 = find(p > p1 - 1e-10);
	b2 = find(p < p2 + 1e-10);
	if ~isempty(b1)
		n1 = b1(1);
	end
	if ~isempty(b2)
		n2 = b2(end);
	end
end
