% calculate V/E/D/Q
function F = vedq(Parm, Mesh, idVolt, Epsr, V, i, j, k)
	% alloc
	F = zeros(1, 10);  % V/E/Ex/Ey/Ez/D/Dx/Dy/Dz/Q

	% mesh
	nx = Mesh.nx;
	ny = Mesh.ny;
	nz = Mesh.nz;
	dxc = Mesh.dxc;
	dyc = Mesh.dyc;
	dzc = Mesh.dzc;
	rxp = Mesh.rxp;
	rxm = Mesh.rxm;
	ryp = Mesh.ryp;
	rym = Mesh.rym;
	rzp = Mesh.rzp;
	rzm = Mesh.rzm;

	% error check
	if (i < 1) || (i > nx) || (j < 1) || (j > ny) || (k < 1) || (k > nz)
		return
	end

	% V [V]
	F(1) = V(i, j, k);

	% E [V/m]
	if (i > 1) && (i < nx)
		F(3) = -(V(i + 1, j, k) - V(i - 1, j, k)) / (2 * dxc(i));
	end
	if (j > 1) && (j < ny)
		F(4) = -(V(i, j + 1, k) - V(i, j - 1, k)) / (2 * dyc(j));
	end
	if (k > 1) && (k < nz)
		F(5) = -(V(i, j, k + 1) - V(i, j, k - 1)) / (2 * dzc(k));
	end
	F(2) = sqrt(F(3)^2 + F(4)^2 + F(5)^2);

	% D [C/m^2]
	F(6:9) = Parm.eps0 * Epsr(i, j, k) * F(2:5);

	% Q [C/m^3]
	if (idVolt(i, j, k) > 0) && (i > 1) && (i < nx) && (j > 1) && (j < ny) && (k > 1) && (k < nz)
		v = V(i, j, k);
		e = Epsr(i, j, k);
		F(10) = Parm.eps0 * ( ...
			(v - V(i + 1, j,     k    )) * (e + Epsr(i + 1, j,     k    )) / 2 * rxp(i) + ...
			(v - V(i - 1, j,     k    )) * (e + Epsr(i - 1, j,     k    )) / 2 * rxm(i) + ...
			(v - V(i,     j + 1, k    )) * (e + Epsr(i,     j + 1, k    )) / 2 * ryp(j) + ...
			(v - V(i,     j - 1, k    )) * (e + Epsr(i,     j - 1, k    )) / 2 * rym(j) + ...
			(v - V(i,     j,     k + 1)) * (e + Epsr(i,     j,     k + 1)) / 2 * rzp(k) + ...
			(v - V(i,     j,     k - 1)) * (e + Epsr(i,     j,     k - 1)) / 2 * rzm(k));
	end
end
