/*
input2.c
*/

#include "ost.h"
#include "ost_prototype.h"

// number of cells
void setup_cells(int nxr, int nyr, int nzr, int *dxr, int *dyr, int *dzr)
{
	int xsum = 0;
	for (int i = 0; i < nxr; i++) {
		xsum += dxr[i];
	}
	Nx = xsum;

	int ysum = 0;
	for (int j = 0; j < nyr; j++) {
		ysum += dyr[j];
	}
	Ny = ysum;

	int zsum = 0;
	for (int k = 0; k < nzr; k++) {
		zsum += dzr[k];
	}
	Nz = zsum;
}

// node
void setup_node(int nxr, int nyr, int nzr, double *xr, double *yr, double *zr, int *dxr, int *dyr, int *dzr)
{
	if ((nxr <= 0) || (nyr <= 0) || (nzr <= 0)) return;

	Xn = (double *)malloc((Nx + 1) * sizeof(double));
	Yn = (double *)malloc((Ny + 1) * sizeof(double));
	Zn = (double *)malloc((Nz + 1) * sizeof(double));

	int xid = 0;
	for (int mx = 0; mx < nxr; mx++) {
		double dx = (xr[mx + 1] - xr[mx]) / dxr[mx];
		for (int i = 0; i < dxr[mx]; i++) {
			Xn[xid++] = xr[mx] + (i * dx);
		}
	}
	Xn[xid] = xr[nxr];

	int yid = 0;
	for (int my = 0; my < nyr; my++) {
		double dy = (yr[my + 1] - yr[my]) / dyr[my];
		for (int j = 0; j < dyr[my]; j++) {
			Yn[yid++] = yr[my] + (j * dy);
		}
	}
	Yn[yid] = yr[nyr];

	int zid = 0;
	for (int mz = 0; mz < nzr; mz++) {
		double dz = (zr[mz + 1] - zr[mz]) / dzr[mz];
		for (int k = 0; k < dzr[mz]; k++) {
			Zn[zid++] = zr[mz] + (k * dz);
		}
	}
	Zn[zid] = zr[nzr];

	// debug
	//for (int i = 0; i <= Nx; i++) printf("Xn[%d]=%.5f\n", i, Xn[i] * 1e3);
	//for (int j = 0; j <= Ny; j++) printf("Yn[%d]=%.5f\n", j, Yn[j] * 1e3);
	//for (int k = 0; k <= Nz; k++) printf("Zn[%d]=%.5f\n", k, Zn[k] * 1e3);
}

// 1d index
void setup_1d(void)
{
	for (int n = 0; n < NL1d; n++) {
		const double p0 = L1d[n].position[0];
		const double p1 = L1d[n].position[1];
		if      (L1d[n].direction == 'X') {
			L1d[n].posid[0] = nearest(p0, 0, Ny, Yn);
			L1d[n].posid[1] = nearest(p1, 0, Nz, Zn);
		}
		else if (L1d[n].direction == 'Y') {
			L1d[n].posid[0] = nearest(p0, 0, Nz, Zn);
			L1d[n].posid[1] = nearest(p1, 0, Nx, Xn);
		}
		else if (L1d[n].direction == 'Z') {
			L1d[n].posid[0] = nearest(p0, 0, Nx, Xn);
			L1d[n].posid[1] = nearest(p1, 0, Ny, Yn);
		}
	}

	// debug
	//for (int n = 0; n < NL1d; n++) printf("1d : %d %c %c %f %d %f %d\n", n, L1d[n].component, L1d[n].direction, pos1[n], L1d[n].posid[0], pos2[n], L1d[n].posid[1]);
}

// near2d index
void setup_2d(void)
{
	for (int n = 0; n < NP2d; n++) {
		const double p = P2d[n].position;
		if      (P2d[n].direction == 'X') {
			P2d[n].posid = nearest(p, 0, Nx, Xn);
		}
		else if (P2d[n].direction == 'Y') {
			P2d[n].posid = nearest(p, 0, Ny, Yn);
		}
		else if (P2d[n].direction == 'Z') {
			P2d[n].posid = nearest(p, 0, Nz, Zn);
		}
	}

	// debug
	//for (int n = 0; n < NP2d; n++) printf("2d : %d %c %c %f %d\n", n, P2d[n].component, P2d[n].direction, pos0[n], P2d[n].posid);
}

// fit geometry without thickness to the nearest node
void fitgeometry(void)
{
	if ((Nx <= 0) || (Ny <= 0) || (Nz <= 0)) return;

	double d0 = EPS * (
		fabs(Xn[Nx] - Xn[0]) +
		fabs(Yn[Ny] - Yn[0]) +
		fabs(Zn[Nz] - Zn[0]));

	for (int n = 0; n < NGeom; n++) {
		int    shape = Geom[n].shape;
		double *g = Geom[n].g;

		if      ((shape == 1) || (shape == 2)) {
			if (fabs(g[0] - g[1]) < d0) {
				int i = nearest(g[0], 0, Nx, Xn);
				g[0] = g[1] = Xn[i];
			}
			if (fabs(g[2] - g[3]) < d0) {
				int j = nearest(g[2], 0, Ny, Yn);
				g[2] = g[3] = Yn[j];
			}
			if (fabs(g[4] - g[5]) < d0) {
				int k = nearest(g[4], 0, Nz, Zn);
				g[4] = g[5] = Zn[k];
			}
		}
		else if ((shape == 11) || (shape == 64) || (shape == 65)) {
			if (fabs(g[0] - g[1]) < d0) {
				int i = nearest(g[0], 0, Nx, Xn);
				g[0] = g[1] = Xn[i];
			}
		}
		else if ((shape == 12) || (shape == 61) || (shape == 66)) {
			if (fabs(g[2] - g[3]) < d0) {
				int j = nearest(g[2], 0, Ny, Yn);
				g[2] = g[3] = Yn[j];
			}
		}
		else if ((shape == 13) || (shape == 62) || (shape == 63)) {
			if (fabs(g[4] - g[5]) < d0) {
				int k = nearest(g[4], 0, Nz, Zn);
				g[4] = g[5] = Zn[k];
			}
		}
	}
}
