/*
plot3dGeom.c

plot geometry 3D
*/

#include "ost.h"
#include "ev.h"
#include "ost_prototype.h"

// geometry
static void plot3dGeom_g(void)
{
	// setup
	int *shape     =         (int *)malloc(NGeom * sizeof(int));
	int *id        =         (int *)malloc(NGeom * sizeof(int));
	double (*g)[8] = (double (*)[8])malloc(NGeom * 8 * sizeof(double));
	for (int n = 0; n < NGeom; n++) {
		shape[n] = Geom[n].shape;
		id[n] = Geom[n].type;
		memcpy(g[n], Geom[n].g, 8 * sizeof(double));
	}

	NGline = geomlines(0, NGeom, shape, NULL, NULL, NULL, NULL, 0);

	Gline  = (double (*)[2][3])malloc(NGline * 2 * 3 * sizeof(double));
	MGline =            (int *)malloc(NGline * sizeof(int));
	memset(Gline, 0, NGline * 2 * 3 * sizeof(double));
	memset(MGline, 0, NGline * sizeof(int));

	const double eps = EPS * (fabs(Xn[Nx] - Xn[0]) + fabs(Yn[Ny] - Yn[0]) + fabs(Zn[Nz] - Zn[0]));
	geomlines(1, NGeom, shape, id, g, Gline, MGline, eps);

	free(shape);
	free(id);
	free(g);

	// dielectrics
	ev3d_setColor(255, 0, 255);
	for (int n = 0; n < NGline; n++) {
		if (MGline[n] == 2) {
			ev3dlib_drawLineA(Gline[n]);
		}
	}

	// electrodes
	ev3d_setColor(0, 0, 0);
	for (int n = 0; n < NGline; n++) {
		if (MGline[n] == 1) {
			ev3dlib_drawLineA(Gline[n]);
		}
	}
}

// mesh
static void plot3dGeom_m(void)
{
	if ((Nx <= 0) || (Ny <= 0) || (Nz <= 0)) return;

	// gray
	ev3d_setColor(200, 200, 200);

	double x1 = Xn[0];
	double x2 = Xn[Nx];
	double y1 = Yn[0];
	double y2 = Yn[Ny];
	double z1 = Zn[0];
	double z2 = Zn[Nz];

	// X constant
	for (int i = 0; i <= Nx; i++) {
		double x = Xn[i];
		ev3d_drawLine(x, y1, z1, x, y2, z1);
		ev3d_drawLine(x, y1, z1, x, y1, z2);
	}

	// Y constant
	for (int j = 0; j <= Ny; j++) {
		double y = Yn[j];
		ev3d_drawLine(x1, y, z1, x1, y, z2);
		ev3d_drawLine(x1, y, z1, x2, y, z1);
	}

	// Z constant
	for (int k = 0; k <= Nz; k++) {
		double z = Zn[k];
		ev3d_drawLine(x1, y1, z, x2, y1, z);
		ev3d_drawLine(x1, y1, z, x1, y2, z);
	}
}

// 1d
static void plot3dGeom_1d(void)
{
	// green
	ev3d_setColor(0, 255, 0);

	for (int n = 0; n < NL1d; n++) {
		if      (L1d[n].direction == 'X') {
			double y = Yn[L1d[n].posid[0]];
			double z = Zn[L1d[n].posid[1]];
			ev3d_drawLine(Xn[0], y, z, Xn[Nx], y, z);
		}
		else if (L1d[n].direction == 'Y') {
			double z = Zn[L1d[n].posid[0]];
			double x = Xn[L1d[n].posid[1]];
			ev3d_drawLine(x, Yn[0], z, x, Yn[Ny], z);
		}
		else if (L1d[n].direction == 'Z') {
			double x = Xn[L1d[n].posid[0]];
			double y = Yn[L1d[n].posid[1]];
			ev3d_drawLine(x, y, Zn[0], x, y, Zn[Nz]);
		}
	}
}

// 2d
static void plot3dGeom_2d(void)
{
	// green
	ev3d_setColor(0, 255, 0);

	for (int n = 0; n < NP2d; n++) {
		if      (P2d[n].direction == 'X') {
			ev3d_drawRectangle('X', Xn[P2d[n].posid], Yn[0], Zn[0], Yn[Ny], Zn[Nz]);
		}
		else if (P2d[n].direction == 'Y') {
			ev3d_drawRectangle('Y', Yn[P2d[n].posid], Zn[0], Xn[0], Zn[Nz], Xn[Nx]);
		}
		else if (P2d[n].direction == 'Z') {
			ev3d_drawRectangle('Z', Zn[P2d[n].posid], Xn[0], Yn[0], Xn[Nx], Yn[Ny]);
		}
	}
}

void plot3dGeom(int ev)
{
	// initialize
	ev3d_init(Width3d, Height3d);

	// new page
	ev3d_newPage();

	// mesh
	plot3dGeom_m();

	// geometry
	plot3dGeom_g();

	// field
	if (runMode == 0) {
		// 1d
		plot3dGeom_1d();
		// 2d
		plot3dGeom_2d();
	}

	// title
	char str[BUFSIZ];
	ev3d_setColor(0, 0, 0);
	ev3d_drawTitle(Fontsize3d, Title);
	sprintf(str, "No. of geometries = %d", NGeom);
	ev3d_drawTitle(Fontsize3d, str);
	sprintf(str, "Nx=%d Ny=%d Nz=%d", Nx, Ny, Nz);
	ev3d_drawTitle(Fontsize3d, str);

	// output
	if (!ev) ev3d_setAngle(Theta3d, Phi3d);
	ev3d_file(ev, (ev ? FN_geom3d_1 : FN_geom3d_0), 0);
	ev3d_output();
}
