/*
vmisc.c
*/

#include "ost.h"

void electrode(int imin, int imax, int jmin, int jmax, int kmin, int kmax)
{
	for (int i = imin; i <= imax; i++) {
	for (int j = jmin; j <= jmax; j++) {
	for (int k = kmin; k <= kmax; k++) {
		const int64_t n = D(i - imin, j - jmin, k - kmin);
		if (idVolt[n]) {
			V[n] = (real_t)Volt[idVolt[n]];
		}
	}
	}
	}
}

void initial(real_t *vmin, real_t *vmax, int imin, int imax, int jmin, int jmax, int kmin, int kmax)
{
	*vmin = +1e10;
	*vmax = -1e10;
	for (int n = 0; n < NGeom; n++) {
		if (Geom[n].type == 1) {
			real_t v = (real_t)Volt[Geom[n].pid];
			*vmin = MIN(*vmin, v);
			*vmax = MAX(*vmax, v);
		}
	}

	if (*vmin == *vmax) return;

	const real_t v0 = (*vmin + *vmax) / 2;
	const real_t f = 2 / (*vmax - *vmin);
	for (int i = imin; i <= imax; i++) {
	for (int j = jmin; j <= jmax; j++) {
	for (int k = kmin; k <= kmax; k++) {
		const int64_t n = D(i - imin, j - jmin, k - kmin);
		if (idVolt[n]) {
			V[n] = f * (V[n] - v0);
		}
		else {
			V[n] = 0;
		}
	}
	}
	}
}

void final(real_t vmin, real_t vmax, int imin, int imax, int jmin, int jmax, int kmin, int kmax)
{
	const real_t v0 = (vmin + vmax) / 2;
	const real_t f = (vmax - vmin) / 2;
	for (int i = imin; i <= imax; i++) {
	for (int j = jmin; j <= jmax; j++) {
	for (int k = kmin; k <= kmax; k++) {
		const int64_t n = D(i - imin, j - jmin, k - kmin);
		V[n] = v0 + (f * V[n]);
	}
	}
	}
}

void edge(int imin, int imax, int jmin, int jmax, int kmin, int kmax)
{
	for (int i = imin; i <= imax; i++) {
	for (int j = jmin; j <= jmax; j++) {
	for (int k = kmin; k <= kmax; k++) {
		const int64_t n = D(i - imin, j - jmin, k - kmin);
		if (!idVolt[n]) {
			int i2 = i - imin;
			int j2 = j - jmin;
			int k2 = k - kmin;
			if ((j ==  0) && (k ==  0)) V[n] = V[D(i2 + 0, j2 + 1, k2 + 1)];
			if ((j == Ny) && (k ==  0)) V[n] = V[D(i2 + 0, j2 - 1, k2 + 1)];
			if ((j ==  0) && (k == Nz)) V[n] = V[D(i2 + 0, j2 + 1, k2 - 1)];
			if ((j == Ny) && (k == Nz)) V[n] = V[D(i2 + 0, j2 - 1, k2 - 1)];
			if ((k ==  0) && (i ==  0)) V[n] = V[D(i2 + 1, j2 + 0, k2 + 1)];
			if ((k == Nz) && (i ==  0)) V[n] = V[D(i2 + 1, j2 + 0, k2 - 1)];
			if ((k ==  0) && (i == Nx)) V[n] = V[D(i2 - 1, j2 + 0, k2 + 1)];
			if ((k == Nz) && (i == Nx)) V[n] = V[D(i2 - 1, j2 + 0, k2 - 1)];
			if ((i ==  0) && (j ==  0)) V[n] = V[D(i2 + 1, j2 + 1, k2 + 0)];
			if ((i == Nx) && (j ==  0)) V[n] = V[D(i2 - 1, j2 + 1, k2 + 0)];
			if ((i ==  0) && (j == Ny)) V[n] = V[D(i2 + 1, j2 - 1, k2 + 0)];
			if ((i == Nx) && (j == Ny)) V[n] = V[D(i2 - 1, j2 - 1, k2 + 0)];
		}
	}
	}
	}
}
